@@ -18,11 +18,13 @@ import logging
 import os
 import prettytable
 import sys
+import tempfile
 import textwrap
 import time
 
 from zuulclient.api import ZuulRESTClient
 from zuulclient.utils import get_default
+from zuulclient.utils import encrypt_with_openssl
 
 
 class ZuulClient():
@@ -84,7 +86,7 @@ class ZuulClient():
         self.add_enqueue_ref_subparser(subparsers)
         self.add_dequeue_subparser(subparsers)
         self.add_promote_subparser(subparsers)
-
+        self.add_encrypt_subparser(subparsers)
         return subparsers
 
     def parseArguments(self, args=None):
@@ -414,6 +416,98 @@ class ZuulClient():
         client = ZuulRESTClient(server, verify, auth_token)
         return client
 
+    def add_encrypt_subparser(self, subparsers):
+        cmd_encrypt = subparsers.add_parser(
+            'encrypt', help='Encrypt a secret to be used in a project\'s jobs')
+        cmd_encrypt.add_argument('--tenant', help='tenant name',
+                                 required=True)
+        cmd_encrypt.add_argument('--project', help='project name',
+                                 required=True)
+        cmd_encrypt.add_argument('--no-strip', action='store_true',
+                                 help='Do not strip whitespace from beginning '
+                                      'or end of input.',
+                                 default=False)
+        cmd_encrypt.add_argument('--secret-name',
+                                 default=None,
+                                 help='How the secret should be named. If not '
+                                      'supplied, a placeholder will be used.')
+        cmd_encrypt.add_argument('--field-name',
+                                 default=None,
+                                 help='How the name of the secret variable. '
+                                      'If not supplied, a placeholder will be '
+                                      'used.')
+        cmd_encrypt.add_argument('--infile',
+                                 default=None,
+                                 help='A filename whose contents will be '
+                                      'encrypted. If not supplied, the value '
+                                      'will be read from standard input.\n'
+                                      'If entering the secret manually, press '
+                                      'Ctrl+d when finished to process the '
+                                      'secret.')
+        cmd_encrypt.add_argument('--outfile',
+                                 default=None,
+                                 help='A filename to which the encrypted '
+                                      'value will be written.  If not '
+                                      'supplied, the value will be written '
+                                      'to standard output.')
+        cmd_encrypt.set_defaults(func=self.encrypt)
+
+    def encrypt(self):
+        if self.args.infile:
+            try:
+                with open(self.args.infile) as f:
+                    plaintext = f.read()
+            except FileNotFoundError:
+                raise Exception('File "%s" not found' % self.args.infile)
+            except PermissionError:
+                raise Exception(
+                    'Insufficient rights to open %s' % self.args.infile)
+        else:
+            plaintext = sys.stdin.read()
+        if not self.args.no_strip:
+            plaintext = plaintext.strip()
+        pubkey_file = tempfile.NamedTemporaryFile(delete=False)
+        self.log.debug('Creating temporary key file %s' % pubkey_file.name)
+        client = self.get_client()
+        try:
+            key = client.get_key(self.args.tenant, self.args.project)
+            pubkey_file.write(str.encode(key))
+            pubkey_file.close()
+
+            self.log.debug('Calling openssl')
+            ciphertext_chunks = encrypt_with_openssl(pubkey_file.name,
+                                                     plaintext,
+                                                     self.log)
+            output = textwrap.dedent(
+                '''
+                - secret:
+                    name: {}
+                    data:
+                      {}: !encrypted/pkcs1-oaep
+                '''.format(self.args.secret_name or '<name>',
+                           self.args.field_name or '<fieldname>'))
+
+            twrap = textwrap.TextWrapper(width=79,
+                                         initial_indent=' ' * 8,
+                                         subsequent_indent=' ' * 10)
+            for chunk in ciphertext_chunks:
+                chunk = twrap.fill('- ' + chunk)
+                output += chunk + '\n'
+
+            if self.args.outfile:
+                with open(self.args.outfile, "w") as f:
+                    f.write(output)
+            else:
+                print(output)
+            return_code = True
+        except Exception as e:
+            self.log.exception(e)
+            return_code = False
+        finally:
+            self.log.debug('Deleting temporary key file %s' % pubkey_file.name)
+            os.unlink(pubkey_file.name)
+        return return_code
+
 
 def main():
     ZuulClient().main()
